// created on 6/14/2004 at 10:39 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 *
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
using Bless.Gui.Drawers;
using Bless.Util; 
using Bless.Buffers;
  
namespace Bless.Gui.Areas {

public abstract class Area {

	protected Gtk.DrawingArea drawingArea;
	protected Drawer drawer;
	protected Drawer.Information drawerInformation;
	protected ByteBuffer byteBuffer;
	
	protected string type;
	
	// display
	protected int x;
	protected int y;
	protected int width;
	protected int height;
	protected long offset;
	protected int bpr;
	protected int dpb; // digits per byte 
	protected int fixedBpr;
	protected Gdk.Drawable backPixmap;
	protected bool manualDoubleBuffer;
	protected bool isAreaRealized;
	
	// GC's
	protected Gdk.GC backEvenGC;
	protected Gdk.GC backOddGC;
	protected Gdk.GC backrEvencSelGC;
	protected Gdk.GC backrOddcSelGC;
	protected Gdk.GC cursorGC;
	
	protected Range selection;
	
	// cursor
	protected long cursorOffset;
	protected int  cursorDigit;
	protected bool cursorFocus;
	protected bool canFocus;

	// track changes
	protected Range prevSelection;  
	protected long prevOffset; 
	protected long prevCursorOffset;
	protected int  prevCursorDigit;
	
	// Abstract methods
	//
	abstract protected void RenderRow(int i, int p, int n, bool blank, bool select);
	
	abstract public void GetDisplayInfoByOffset(long off, out int orow, out int obyte, out int ox, out int oy);
	
	public enum GetOffsetFlags { Eof=1, Abyss=2}
	
	abstract public long GetOffsetByDisplayInfo(int x, int y,out int digit, out GetOffsetFlags rflags);
	
	virtual public bool HandleKey(Gdk.Key key, bool overwrite)
	{
		return false;
	}
	

	///<summary>
	/// Calculates the width in pixels the    
	/// area will occupy with n bytes per row.
	///</summary>
	abstract public int CalcWidth(int n);
	
	static public Area Factory(string name)
	{
		Area a;
		
		switch (name) {
			case "hexadecimal":
				a=new HexArea();
				break;
			case "binary":
				a=new BinaryArea();
				break;
			case "decimal":
				a=new DecimalArea();
				break;
			case "octal":
				a=new OctalArea();
				break;
			case "ascii":
				a=new AsciiArea();
				break;
			case "separator":
				a=new SeparatorArea();
				break;
			case "offset":
				a=new OffsetArea();
				break;				
			default:
				a=null;
				break;
		}	
		
		return a;
	
	}
	
	///<summary>
	/// Create an area.
	///</summary>
	public Area() 
	{
		selection=new Range();
		prevSelection=new Range();
		
		canFocus=false;
		dpb=1024; // arbitrary large int
		fixedBpr=-1;
		isAreaRealized=false;
	}
	
	///<summary>
	/// Realize the area.
	///</summary>
	public virtual void Realize(Gtk.DrawingArea da, Drawer d)
	{
		drawingArea=da;
		
		backPixmap=da.GdkWindow;
		backEvenGC=new Gdk.GC(da.GdkWindow);
		backOddGC=new Gdk.GC(da.GdkWindow);
		backrOddcSelGC=new Gdk.GC(da.GdkWindow);
		backrEvencSelGC=new Gdk.GC(da.GdkWindow);
		cursorGC=new Gdk.GC(da.GdkWindow);
		 
		drawer=d; 
		Gdk.Color col=new Gdk.Color();
		Gdk.Color.Parse(drawer.Info.rEvencEvenBg, ref col); 
		backEvenGC.RgbFgColor=col;
		Gdk.Color.Parse(drawer.Info.rOddcEvenBg, ref col); 
		backOddGC.RgbFgColor=col;
		Gdk.Color.Parse(drawer.Info.rEvencSelBg, ref col); 
		backrEvencSelGC.RgbFgColor=col;
		Gdk.Color.Parse(drawer.Info.rOddcSelBg, ref col); 
		backrOddcSelGC.RgbFgColor=col;
		Gdk.Color.Parse("red", ref col); 
		cursorGC.RgbFgColor=col;
		
		isAreaRealized=true;
	}
	
	// Properties
	//
	public int X {
		set { x=value; }
		get	{ return x; }
	} 

	public int Y {
		set { y=value; }
		get	{ return y; }
	}
	
	public int Width {
		set { width=value; }
		get	{ return width; }
	} 

	public int Height {
		set { height=value; }
		get	{ return height; }
	}
	
	public int BytesPerRow {
		set { bpr=value; }
		get	{ return bpr; }
	}
	
	public int FixedBytesPerRow {	
		set { fixedBpr=value; }
		get { return fixedBpr; }
	}
	
	public int DigitsPerByte {
		get	{ return dpb; }
	}
	
	public long Offset {
		set { prevOffset=offset; offset=value; } 
		get { return offset;}
	}

	public long CursorOffset {
		set { prevCursorOffset=cursorOffset; cursorOffset=value;} 
		get { return cursorOffset;}
	}
	
	public long PrevCursorOffset { 
		get { return prevCursorOffset;}
	}
	
	public int CursorDigit {
		set { prevCursorDigit=cursorDigit; if (value >= dpb) cursorDigit=dpb-1; else cursorDigit=value;} 
		get { return cursorDigit;}
	}
	
	public bool HasCursorFocus {
		set { cursorFocus=value; CursorDigit=-1;} 
		get { return cursorFocus;}
	}
	
	public bool CanFocus { 
		get { return canFocus;}
	}
	
	public Drawer.Information DrawerInformation
	{
		get { return drawerInformation; }
		set { drawerInformation=value; }
	}
	
	public virtual Drawer Drawer {
		get { return drawer; }
	}

	public string Type {
		get { return type; }
	}
	
	public Gtk.DrawingArea DrawingArea {
		get { return drawingArea; }
	}
	
	public ByteBuffer Buffer {
		get { return byteBuffer; }
		set { byteBuffer=value; }
	}
	
	public Range Selection {
		get { return selection; }
		set { selection=value; }
	} 
	
	
	///<summary>
	/// Render the 'selection' taking
	/// into account differences from the 'prevSelection'
	///</summary>	
	protected void RenderSelectionDiffs(Range selection, Range prevSelection, Range clip)
	{
		if (isAreaRealized==false)
			return;
		
		//BeginPaint();
		// find what parts of the selection have changed
		Range added=new Range(selection);
		added.Sort();
		
		// find unchanged range		
		Range common=new Range(prevSelection);
		common.Sort();
		common.Intersect(added);
		common.Intersect(clip);
			
		// find lost (now unselected) ranges
		Range lost1=new Range(prevSelection);
		Range lost2=new Range();
		lost1.Sort();
		lost1.Difference(added, lost2);
		lost1.Intersect(clip);
		lost2.Intersect(clip);
			
		// find added ranges
		Range old=new Range(prevSelection);
		Range added1=new Range();
		old.Sort();
		added.Difference(old, added1);
		added.Intersect(clip);
		
		//System.Console.WriteLine("RenderSelectionDiffs: Clip=({0},{1}) Added=({2},{3}) Lost1=({4},{5}) Lost2=({6},{7}) Common=({8},{9}) PrevSel=({10},{11}) CurSel=({12},{13})",
		//						clip.Start, clip.End, added.Start, added.End, lost1.Start, lost1.End, lost2.Start, lost2.End, common.Start, common.End, prevSelection.Start, prevSelection.End, selection.Start, selection.End );
		//System.Console.WriteLine("Common: {0} {1}",common.Start, common.End);
		//System.Console.WriteLine("Lost: {0} {1}",lost.Start, lost.End);
		//System.Console.WriteLine("Added: {0} {1}",added.Start, added.End);
			
		// render the lost parts normally
		int rLost1Start=-1;
		int rLost1End=-1;
		int rLost2Start=-1;
		int rLost2End=-1;
		
		if (!lost1.IsEmpty()) {
			rLost1Start=((int)(lost1.Start-offset)/bpr);
			rLost1End=((int)(lost1.End-offset)/bpr);
			//render the lines fully
			for (int j=rLost1Start; j<rLost1End; j++)
				RenderRow(j, 0, bpr, true, false);
			// make sure we are ot out of range at the last line
			long len = byteBuffer.Size-(lost1.End/bpr)*bpr;
            if (len > bpr) len=bpr;
			RenderRow(rLost1End, 0, (int)len, true, false);
		}
	
		if (!lost2.IsEmpty()) {
			rLost2Start=((int)(lost2.Start-offset)/bpr);
			rLost2End=((int)(lost2.End-offset)/bpr);
			//render the middle lines fully
			for (int j=rLost2Start; j<rLost2End; j++)
				RenderRow(j, 0, bpr, true, false);
			// make sure we are ot out of range at the last line
			long len = byteBuffer.Size-(lost2.End/bpr)*bpr;
            if (len > bpr) len=bpr;
			RenderRow(rLost2End, 0, (int)len, true, false);
        }
		
		
		// re-render the common parts that overlap the lost
		// parts as selections
		if (!common.IsEmpty()) {
			int rCommonStart=((int)(common.Start-offset)/bpr);
			int rCommonEnd=((int)(common.End-offset)/bpr);
			if (rCommonStart == rLost1End && rCommonEnd == rLost1End
				&& rLost1End == rLost1Start) {
				RenderSelection(common);
			}
			else if (rCommonStart == rLost2End && rCommonEnd == rLost2End
				&& rLost2End == rLost2Start) {
				RenderSelection(common);
			}
			else if (rCommonStart == rLost1End || rCommonStart == rLost2End) {
				int nbyte=(int)((common.Start-offset)%bpr);
				long overLeft = common.End-common.Start < bpr-nbyte ? common.End-common.Start:bpr-nbyte;
				Range rOverlap=new Range(common.Start, common.Start+overLeft);
				RenderSelection(rOverlap);
			} 
			else if (rCommonEnd == rLost1Start || rCommonEnd == rLost2Start) {
				int nbyte=(int)((common.End-offset)%bpr);
				long rCommonEndOffset=rCommonEnd*bpr+offset;
				long overStart=rCommonEndOffset > common.Start? rCommonEndOffset:common.Start;
				// part of common that is on row rCommonEnd
				Range rOverlap=new Range(overStart, rCommonEndOffset+nbyte);
				RenderSelection(rOverlap);
			}
		}
		
		
		// render the added parts as selections
		if (!added.IsEmpty()) {
			if ((added.Start==common.End+1) && added.Start>0)
				--added.Start;
			if (added.End==common.Start-1)
				++added.End;
			RenderSelection(added);
		}
		
		if (!added1.IsEmpty()) {
			if ((added1.Start==common.End+1) && added1.Start>0)
				--added1.Start;
			if (added1.End==common.Start-1)
				++added1.End;
			RenderSelection(added1);
		}
		
		//EndPaint();
			
	}
	
	///<summary>
	/// Render the bytes in 'range' 
	/// 'select' is true if bytes should be rendered as selected 
	///</summary>
	protected void RenderRange(Range range, bool select)
	{
		if (isAreaRealized==false)
			return;
		
		int rstart, bstart, xstart, ystart;
		int rend, bend, xend, yend;
		bool odd;
		Gdk.GC gc;
		Gdk.GC oddGC;
		Gdk.GC evenGC;
		
		// choose GCs
		if (select) {
			oddGC=backrOddcSelGC;
			evenGC=backrEvencSelGC;
		}
		else  {
			oddGC=backOddGC;
			evenGC=backEvenGC;
		}
			
			
		GetDisplayInfoByOffset(range.Start,out rstart, out bstart, out xstart, out ystart);
		GetDisplayInfoByOffset(range.End,out rend, out bend, out xend, out yend);

		//System.Console.WriteLine("Start {0:x} {1} {2} x:{3} y:{4}", sel.Start, rstart, bstart, xstart, ystart);
		//System.Console.WriteLine("End {0:x} {1} {2} x:{3} y:{4}", sel.End, rend, bend, xend, yend);
		
		// if the whole range is on one row 
		if (rstart==rend) {
			if (manualDoubleBuffer)
				BeginPaint(x+xstart, y+ystart, xend-xstart+dpb*drawer.Width, drawer.Height);
			
			// odd row?
			odd=(((range.Start/bpr)%2)==1);
			if (odd)
				gc=oddGC;
			else	
				gc=evenGC;
			//render
			backPixmap.DrawRectangle(gc, true, x+xstart, y+ystart, xend-xstart, drawer.Height);
			RenderRow(rstart, bstart, bend-bstart+1, false, select);
		}
		else { // multi-row range
			
			if (manualDoubleBuffer) {
				// handle double-buffering
				Gdk.Region paintRegion=new Gdk.Region();
			
				Gdk.Rectangle rectStart=new Gdk.Rectangle(x+xstart, y+ystart, width-xstart, drawer.Height);
			
				Gdk.Rectangle rectMiddle;
				if (rend > rstart + 1)
					rectMiddle=new Gdk.Rectangle(x, y+ystart+drawer.Height, width, yend-ystart-drawer.Height);
				else
					rectMiddle=Gdk.Rectangle.Zero;
			
				Gdk.Rectangle rectEnd=new Gdk.Rectangle(x, y+yend, xend+dpb*drawer.Width, drawer.Height);
		
				paintRegion.UnionWithRect(rectStart);
				paintRegion.UnionWithRect(rectMiddle);
				paintRegion.UnionWithRect(rectEnd);
			
				BeginPaintRegion(paintRegion);
			}
			
			// render first row
			odd=(((range.Start/bpr)%2)==1);
			if (odd)
				gc=oddGC;
			else	
				gc=evenGC;
			backPixmap.DrawRectangle(gc, true, x+xstart, y+ystart, width-xstart, drawer.Height);
			RenderRow(rstart, bstart, bpr-bstart, false, select);
			
			long curOffset=range.Start+bpr-bstart;
			
			// render middle rows
			for(int i=rstart+1;i<rend;i++) {
				odd=(((curOffset/bpr)%2)==1);
				if (odd)
					gc=oddGC;
				else	
					gc=evenGC;
				backPixmap.DrawRectangle(gc, true, x, y+i*drawer.Height, width, drawer.Height);
				RenderRow(i, 0, bpr, false, select);
				curOffset+=bpr;
			}
			
			// render last row
			odd=(((range.End/bpr)%2)==1);
			if (odd)
				gc=oddGC;
			else	
				gc=evenGC;
			backPixmap.DrawRectangle(gc, true, x, y+yend, xend, drawer.Height);
			RenderRow(rend, 0, bend+1, false, select);
		}
		
		if (manualDoubleBuffer)
			EndPaint();
		
	}
	
	///<summary>Render a range of bytes as selected</summary>
	protected void RenderSelection(Range sel)
	{
		RenderRange(sel, true);
	}
	
	///<summary>Render a range of bytes normally</summary>
	protected void RenderNormally(Range r)
	{
		RenderRange(r, false);
	}
	
	///<summary>Render the cursor</summary>
	protected void RenderCursor()
	{
		if (isAreaRealized==false)
			return;
		
		// don't draw the cursor when there is a selection
		if (!selection.IsEmpty())
			return;
			
		int cRow, cByte, cX, cY;
		GetDisplayInfoByOffset(cursorOffset, out cRow, out cByte, out cX, out cY);
	
		backPixmap.DrawRectangle(cursorGC, true, x+cX, y+cY+drawer.Height-2, drawer.Width*dpb, 2);
		if (cursorFocus) {
			backPixmap.DrawRectangle(cursorGC, true, x+cX+cursorDigit*drawer.Width, y+cY, 1, drawer.Height-2);
		}
	}
	
	public void DisposePixmaps()
	{
		if (isAreaRealized==false)
			return;
			
		backPixmap.Dispose();
		drawer.DisposePixmaps();
	}
	
	void BeginPaintRegion(Gdk.Region r)
	{
		Gdk.Window win=drawingArea.GdkWindow;
		
		win.BeginPaintRegion(r);
	}
	
	void BeginPaint()
	{
		BeginPaint(this.x, this.y, this.width, this.height);
	}
	
	void BeginPaint(int x, int y, int w, int h)
	{
		Gdk.Window win=drawingArea.GdkWindow;
		
		win.BeginPaintRect(new Gdk.Rectangle(x, y, w, h));
	}
	
	void EndPaint()
	{
		drawingArea.GdkWindow.EndPaint();
	}
	
	
	///
	/// Public Interface
	///
	
	///<summary>Draws the area</summary>
	public virtual void Render() 
	{		
		// if we don't have enough space to draw return 
		if (bpr<=0)
			return;

		Scroll(offset);
	}
	
	///<summary>Moves and draws the cursor</summary>
	public virtual void MoveCursor(long offset, int digit)
	{
		RenderOffset(cursorOffset);
		this.CursorOffset=offset;
		this.CursorDigit=digit;
		RenderCursor();
	}
	
	///<summary>Moves but does not draw the cursor</summary>
	public virtual void MoveCursorNoRender(long offset, int digit)
	{
		this.CursorOffset=offset;
		this.CursorDigit=digit;
	}
	
	///<summary>Renders a single offset</summary>
	public virtual void RenderOffset(long offs)
	{
		if (isAreaRealized==false)
			return;
		
		int nrows=height/drawer.Height;
		long bleft=nrows*bpr;
        
		if (bleft+offset >= byteBuffer.Size)
			bleft=byteBuffer.Size - offset;
			
		if (offs >= offset && offs < offset+bleft) {	 
			int pcRow, pcByte, pcX, pcY;
			GetDisplayInfoByOffset(offs, out pcRow, out pcByte, out pcX, out pcY);
			RenderRow(pcRow, pcByte, 1, false, selection.Contains(offs));	
		}
		else if (offs == byteBuffer.Size && offs == offset+bleft) {
			int pcRow, pcByte, pcX, pcY;
			GetDisplayInfoByOffset(offs, out pcRow, out pcByte, out pcX, out pcY);
			backPixmap.DrawRectangle(backEvenGC, true, x+pcX, y+pcY, drawer.Width*dpb, drawer.Height);
		}

	}
	
	
	public virtual void RenderBackground(Drawer.RowType rtype, Drawer.ColumnType ctype, int x, int y, int w, int h)
	{
	
	
	}
	
	///<summary>Scrolls the view so that 'offset' is the first visible offset</summary>
	public virtual void Scroll(long offset)
	{	
		this.prevOffset=this.offset;
		this.offset=offset;
		
		if (isAreaRealized==false)
			return;
		
		// find out number of rows, bytes etc
		int nrows=height/drawer.Height;
		long bleft=nrows*bpr;
    
		if (bleft+offset >= byteBuffer.Size)
			bleft=byteBuffer.Size - offset;
                                                                                
		// blank the background
		backPixmap.DrawRectangle(backEvenGC, true, x,y, width, height);

		Range rSel=new Range(selection);
		rSel.Sort();
		
		Range rClip;
		// make sure we get an empty clipping Range when bleft==0 
		if (bleft>0)
			rClip=new Range(offset, offset+bleft-1);
		else
			rClip=new Range();

		// calculate the ranges to render normally (not selected)
		Range rNormal1=new Range(rClip);
		Range rNormal2=new Range();
		rNormal1.Difference(rSel, rNormal2);
		
		if (!rNormal1.IsEmpty())
			RenderNormally(rNormal1);
		if (!rNormal2.IsEmpty())
			RenderNormally(rNormal2);
				
		// render selection
		rSel.Intersect(rClip);

		if (!rSel.IsEmpty())
			RenderSelection(rSel);
			
		// render the cursor	
		if ((cursorOffset >= offset && cursorOffset < offset+bleft)
			|| (cursorOffset == byteBuffer.Size && cursorOffset == offset+bleft))
			RenderCursor();
		
	}
	
	///<summary>Set and draw the selection</summary>
	public virtual void SetSelection(long start, long end)
	{
		if (start==selection.Start && end==selection.End)
			return;
			
		manualDoubleBuffer=true;
		
		selection.Start=start;
		selection.End=end;
				
		int nrows=height/drawer.Height;
		long bleft=nrows*bpr;
		
		if (bleft+offset >= byteBuffer.Size)
			bleft=byteBuffer.Size - offset;
		
		Range clip;
						
		if (bleft>0)
			clip=new Range(offset, offset+bleft-1);
		else
			clip=new Range();
			
		RenderSelectionDiffs(selection, prevSelection, clip);
		//Scroll(offset);

		prevSelection.Start=start;
		prevSelection.End=end;
		
		manualDoubleBuffer=false;
	} 
	
	///<summary>Set but don't draw the selection</summary>
	public virtual void SetSelectionNoRender(long start, long end)
	{
		selection.Start=start;
		selection.End=end;
					
		prevSelection.Start=start;
		prevSelection.End=end;
	}
	
	
}// Area


} //namespace
