// created on 6/14/2004 at 10:55 PM
/*
 *   Copyright (c) 2004, Alexandros Frantzis (alf82 [at] freemail [dot] gr)
 * 
 *   This file is part of Bless.
 *
 *   Bless is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Bless is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Bless; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using Gtk;
using Gdk;
using Pango;

namespace Bless.Gui.Drawers {

///<summary>Fast font drawing class</summary>
public abstract class Drawer {

	public class Information {
		public string FontName;
		public string FontLanguage;
		
		public string rEvencEvenFg;
		public string rEvencEvenBg;
		public string rEvencOddFg;
		public string rEvencOddBg;
		public string rEvencSelFg;
		public string rEvencSelBg;
		
		public string rOddcEvenFg;
		public string rOddcEvenBg;
		public string rOddcOddFg;
		public string rOddcOddBg;
		public string rOddcSelFg;
		public string rOddcSelBg;
		
		
		public bool Uppercase;
		
		public Information() 
		{	
			FontName="Courier 12";
			FontLanguage="utf-8";
			
			rEvencEvenFg="black";
			rEvencEvenBg="white";
			
			rEvencOddFg="blue";
			rEvencOddBg="white";
			
			
			rOddcEvenFg="black";
			rOddcEvenBg="white";
			
			rOddcOddFg="blue";
			rOddcOddBg="white";
			
			// leave unspecified...
			// if not specified by user they will
			// be set up using theme defaults
			rEvencSelFg=null;
			rEvencSelBg=null;
			
			rOddcSelFg=null;
			rOddcSelBg=null;
			
			
			Uppercase=false;
		}
		
		// setup unspecified selection colors using theme default colors
		public void SetupSelection(Gtk.Widget widget)
		{
			Gdk.Color col;
			
			col=widget.Style.TextColors[(int)StateType.Selected];
			string strFg=string.Format("#{0:x2}{1:x2}{2:x2}", (byte)col.Red,(byte)col.Green,(byte)col.Blue);
			
			col=widget.Style.BaseColors[(int)StateType.Selected];
			string strBg=string.Format("#{0:x2}{1:x2}{2:x2}", (byte)col.Red,(byte)col.Green,(byte)col.Blue);
			
			if (rEvencSelFg==null)
				rEvencSelFg=strFg;
			
			if (rEvencSelBg==null)
				rEvencSelBg=strBg;
			
			if (rOddcSelFg==null)
				rOddcSelFg=strFg;
			
			if (rOddcSelBg==null)
				rOddcSelBg=strBg;
		}
		
		
	}
	
	public enum RowType {Even, Odd};
	public enum ColumnType {Even, Odd, Selected};
	
	// the widget the font will finally 
	// be printed on (used for info only)
	protected Gtk.Widget widget;
	protected Pango.FontDescription fontDescription;
	protected Information info;
	
	protected Gdk.Pixmap rEvencEven;
	protected Gdk.Pixmap rEvencOdd;
	protected Gdk.Pixmap rEvencSel;
	
	protected Gdk.Pixmap rOddcEven;
	protected Gdk.Pixmap rOddcOdd;
	protected Gdk.Pixmap rOddcSel;
	
	protected int width;
	protected int height;
	
	///<summary>Constructor</summary>
	public Drawer(Gtk.Widget wid, Information inf)
	{
		widget=wid;
		info=inf;
		
		// make sure selection colors are set
		info.SetupSelection(wid);
		
		fontDescription=Pango.FontDescription.FromString(info.FontName);
		Pango.Language lang=Pango.Language.FromString(info.FontLanguage);
		
		Pango.Context pangoCtx=widget.PangoContext;
		Pango.FontMetrics fm=pangoCtx.GetMetrics(fontDescription, lang);
		
		// set the font height and width
		height =(int)( (fm.Ascent+fm.Descent)/Pango.Scale.PangoScale);
    	width =(int) ((fm.ApproximateCharWidth)/Pango.Scale.PangoScale);

		// create the font pixmaps
		Gdk.Color colorFg=new Gdk.Color();
		Gdk.Color colorBg=new Gdk.Color();
		
		//even rows
		Gdk.Color.Parse(info.rEvencEvenFg, ref colorFg);
		Gdk.Color.Parse(info.rEvencEvenBg, ref colorBg);
		rEvencEven=Create(colorFg, colorBg);
		
		Gdk.Color.Parse(info.rEvencOddFg, ref colorFg);
		Gdk.Color.Parse(info.rEvencOddBg, ref colorBg);
		rEvencOdd=Create(colorFg, colorBg);
		
		Gdk.Color.Parse(info.rEvencSelFg, ref colorFg);
		Gdk.Color.Parse(info.rEvencSelBg, ref colorBg);
		rEvencSel=Create(colorFg, colorBg);
		
		//odd rows
		Gdk.Color.Parse(info.rOddcEvenFg, ref colorFg);
		Gdk.Color.Parse(info.rOddcEvenBg, ref colorBg);
		rOddcEven=Create(colorFg, colorBg);
				
		Gdk.Color.Parse(info.rOddcOddFg, ref colorFg);
		Gdk.Color.Parse(info.rOddcOddBg, ref colorBg);
		rOddcOdd=Create(colorFg, colorBg);
		
		Gdk.Color.Parse(info.rOddcSelFg, ref colorFg);
		Gdk.Color.Parse(info.rOddcSelBg, ref colorBg);
		rOddcSel=Create(colorFg, colorBg);

	}
	
	///<summary>Creates a pixmap with the drawn data</summary>
	abstract protected Gdk.Pixmap Create(Gdk.Color fg, Gdk.Color bg);
	
	///<summary>Draws the a byte</summary>
	abstract public void Draw(Gdk.GC gc, Gdk.Drawable dest, int x, int y, byte b, RowType rtype, ColumnType ctype); 
 
	///<summary>Select the pixmap according to the type</summary>
	protected Gdk.Pixmap SelectPixmap(RowType rtype, ColumnType ctype)
	{
		Gdk.Pixmap pix=null;
 
		if (rtype == RowType.Even) {
			if (ctype==ColumnType.Even) 
				pix=rEvencEven;
			else if (ctype==ColumnType.Odd)
				pix=rEvencOdd;
			else /*if (ctype==ColumnType.Selected)*/
				pix=rEvencSel;
		}
		else if (rtype == RowType.Odd) {
			if (ctype==ColumnType.Even) 
				pix=rOddcEven;
			else if (ctype==ColumnType.Odd)
				pix=rOddcOdd;
			else /*if (ctype==ColumnType.Selected)*/
				pix=rOddcSel;
		}
		
		return pix;
	}
	
	public void DisposePixmaps()
	{
		if (rEvencEven != null)
			rEvencEven.Dispose();
		if (rEvencOdd != null)
			rEvencOdd.Dispose();
		if (rEvencSel != null)
			rEvencSel.Dispose();
	
		if (rOddcEven != null)
			rOddcEven.Dispose();
		if (rOddcOdd != null)
			rOddcOdd.Dispose();
		if (rOddcSel != null)
			rOddcSel.Dispose();
	}
	
	public int Width{
		get { return width; }
	}
	
	public int Height{
		get { return height; }
	}
	
	public Drawer.Information Info{
		get { return info; }
	}

}

//<summary>dummy</summary>
public class DummyDrawer : Drawer {
	
	public DummyDrawer(Gtk.Widget wid, Information inf)
	:base(wid, inf)
	{
	}
	
	public override void Draw(Gdk.GC gc, Gdk.Drawable dest, int x, int y, byte b, RowType rtype, ColumnType ctype) 
	{
		
	}
	
	protected override Gdk.Pixmap Create(Gdk.Color fg, Gdk.Color bg)
	{
		return null;
	}
	
	

}

} // end namespace
